package hamster;

/**
 * Gegeben ein Territorium ohne innere Mauern. Dann sammelt ein Hamster der
 * Klasse AbraeumerHamster alle Koerner des Territoriums auf.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (25.01.2006)
 * 
 */
public class AbraeumerHamster extends AllroundHamster {

	/**
	 * initialisiert einen neuen AbraeumerHamster mit den uebergebenen Werten
	 * 
	 * @param reihe
	 *            Reihe der Kachel
	 * @param spalte
	 *            Spalte der Kachel
	 * @param blickrichtung
	 *            Blickrichtung des Hamsters
	 * @param anzahlKoerner
	 *            Anzahl an Koernern im Maul des Hamsters
	 */
	public AbraeumerHamster(int reihe, int spalte, int blickrichtung,
			int anzahlKoerner) {
		super(reihe, spalte, blickrichtung, anzahlKoerner);
	}

	/**
	 * laedt ein vordefiniertes Territorium und erzeugt und initialisiert darin
	 * einen neuen AbraeumerHamster mit den uebergebenen Werten
	 * 
	 * @param reihe
	 *            Reihe der Kachel
	 * @param spalte
	 *            Spalte der Kachel
	 * @param blickrichtung
	 *            Blickrichtung des Hamsters
	 * @param anzahlKoerner
	 *            Anzahl an Koernern im Maul des Hamsters
	 * @param territoriumsDatei
	 *            Name einer Datei mit einem zu ladenden Territorium
	 */
	public AbraeumerHamster(int reihe, int spalte, int blickrichtung,
			int anzahlKoerner, String territoriumsDatei) {
		super(reihe, spalte, blickrichtung, anzahlKoerner, territoriumsDatei);
	}

	/**
	 * laedt ein vordefiniertes Territorium und initialisiert einen neuen
	 * AbraeumerHamster mit den Attributwerten eines bereits existierenden
	 * Hamsters
	 * 
	 * @param existierenderHamster
	 *            existierender Hamster
	 * @param territoriumsDatei
	 *            Name einer Datei mit einem zu ladenden Territorium
	 */
	public AbraeumerHamster(Hamster existierenderHamster,
			String territoriumsDatei) {
		super();
		Territorium.ladeTerritorium(territoriumsDatei);
		this.init(existierenderHamster.getReihe(), existierenderHamster
				.getSpalte(), existierenderHamster.getBlickrichtung(),
				existierenderHamster.getAnzahlKoerner());
	}

	/**
	 * der Hamster sammelt alle Koerner des Territoriums auf. Voraussetzung: Im
	 * Territorium befinden sich keine inneren Mauern.
	 */
	public void territoriumLeeren() {
		this.laufeZuKachel(0, 0);
		this.setzeBlickrichtung(hamster.Hamster.OST);
		this.reiheLeeren();
		while (weitereReiheExistiert()) {
			this.begibDichInNaechsteReihe();
			this.reiheLeeren();
		}
	}

	private void reiheLeeren() {
		this.nimmAlle();
		while (this.vornFrei()) {
			this.vor();
			this.nimmAlle();
		}
	}

	private boolean weitereReiheExistiert() {
		if (this.getBlickrichtung() == Hamster.OST) {
			return this.rechtsFrei();
		} else {
			return this.linksFrei();
		}
	}

	private void begibDichInNaechsteReihe() {
		if (this.getBlickrichtung() == Hamster.OST) {
			this.rechtsUm();
			this.vor();
			this.rechtsUm();
		} else {
			this.linksUm();
			this.vor();
			this.linksUm();
		}
	}

}
