package hamster;

/**
 * Ein Hamster, der beim NimmSpiel mitspielen will, muss von dieser Klasse
 * abgeleitet werden.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (28.01.2006)
 * 
 */
public abstract class SpielHamster extends AllroundHamster {

	protected int laengeDerKornReihe;

	/**
	 * Konstruktor zum Erzeugen und Initialisieren eines Hamsters mit den
	 * uebergebenen Parametern
	 * 
	 * @param reihe
	 *            die Reihe des Territoriums, in der der Hamster erzeugt wird
	 * @param spalte
	 *            die Spalte des Territoriums, in der der Hamster erzeugt wird
	 * @param blickrichtung
	 *            die Richtung, in der der Hamster anfangs schaut (siehe
	 *            Konstanten)
	 * @param anzahlKoerner
	 *            die Anzahl an Koernern, die der Hamster anfangs im Maul hat
	 */
	public SpielHamster(int reihe, int spalte, int blickrichtung,
			int koernerAnzahl) {
		super(reihe, spalte, blickrichtung, koernerAnzahl);
		this.laengeDerKornReihe = 0;
	}

	/**
	 * der Hamster begibt sich zum Start eines Spiels
	 * 
	 * @param reihe
	 *            Startreihe
	 * @param spalte
	 *            Startspalte
	 * @param blickrichtung
	 *            Blickrichtung beim Start
	 */
	public void begibDichZumStart(int reihe, int spalte, int blickrichtung) {
		this.laufeZuKachel(reihe, spalte);
		this.setzeBlickrichtung(blickrichtung);
	}

	/**
	 * Teilt dem Hamster die Laenge der Kornreihe mit
	 * 
	 * @param laenge
	 *            Laenge der Kornreihe
	 */
	public void setLaengeDerKornReihe(int laenge) {
		this.laengeDerKornReihe = laenge;
	}

	/**
	 * die Methode zum Fressen; uebergeben wird, wie viele Koerner der Gegner
	 * gefressen hat; geliefert wird, wie viele Koerner der Hamster gefressen
	 * hat
	 * 
	 * @param gegnerAnzahl
	 *            Anzahl der vom Gegner zuvor gefressenen Koerner
	 * @return Anzahl selbst gefressener Koerner
	 */
	public abstract int naechsterSpielzug(int gegnerAnzahl);

}
